@@ -111,7 +111,7 @@ class InAgentClient(BaseInAgentClient):
                                                     continue
                                                 t, is_new = await models.Transfer.update_or_create(
                                                     dict(
-                                                        amount=int(float(order.amount)) * 100,
+                                                        amount=int(float(order.amount) * 100),
                                                         accepted_at=datetime.now(timezone.utc),
                                                     ),
                                                     order=order_db,
@@ -126,7 +126,7 @@ class InAgentClient(BaseInAgentClient):
                                                         addr__coin_id=order_db.ad.pair_side.pair.coin_id,
                                                         addr__actor=self.agent_client.actor,
                                                     )
-                                                    ass.free -= float(order_db.amount / order_db.ad.price)
+                                                    ass.free -= int(order_db.amount / order_db.ad.price)
                                                     await ass.save(update_fields=["free"])
                                                     # обновляем остаток валюты
                                                     fiat = await models.Fiat.get(cred_id=order_db.cred_id)
@@ -247,7 +247,7 @@ class InAgentClient(BaseInAgentClient):
     async def send_payment(self, order_db: models.Order, dest):
         if order_db.status not in (OrderStatus.created, OrderStatus.requested):
             return
-        int_am = round(order_db.amount * 10**-2, 2)
+        fmt_am = round(order_db.amount * 10**-2, 2)
         pma, cur = await self.get_pma_by_pmex(order_db)
         async with in_transaction():
             # отмечаем ордер на бирже "оплачен"
@@ -259,9 +259,9 @@ class InAgentClient(BaseInAgentClient):
                 paymentId=str(credex.exid),  # credex.exid
             )
             # проверяем не отправляли ли мы уже перевод по этому ордеру
-            if t := await models.Transfer.get_or_none(order=order_db, amount=int_am):
-                await pma.bot.send(f"Order# {order_db.exid}: Double send {int_am}{cur} to {dest} #{t.pmid}!")
-                raise Exception(f"Order# {order_db.exid}: Double send {int_am}{cur} to {dest} #{t.pmid}!")
+            if t := await models.Transfer.get_or_none(order=order_db, amount=order_db.amount):
+                await pma.bot.send(f"Order# {order_db.exid}: Double send {fmt_am}{cur} to {dest} #{t.pmid}!")
+                raise Exception(f"Order# {order_db.exid}: Double send {fmt_am}{cur} to {dest} #{t.pmid}!")
 
             # ставим в бд статус "оплачен"
             order_db.status = OrderStatus.paid
@@ -271,24 +271,24 @@ class InAgentClient(BaseInAgentClient):
             ass = await models.Asset.get(
                 addr__coin_id=order_db.ad.pair_side.pair.coin_id, addr__actor=self.agent_client.actor
             )
-            ass.free += float(order_db.amount / order_db.ad.price)
+            ass.free += int(order_db.amount / order_db.ad.price)
             await ass.save(update_fields=["free"])
             # отправляем деньги
-            tid, img, rest_amount = await pma.send(dest=dest, amount=int_am, cur=cur)
+            tid, img, rest_amount = await pma.send(dest=dest, amount=fmt_am, cur=cur)
             # создаем перевод в бд
             t, _ = await models.Transfer.update_or_create(
-                {"amount": int_am * 100},
+                {"amount": order_db.amount},
                 order=order_db,
                 pmid=tid,
             )
             # обновляем остаток валюты
-            await models.Fiat.update_or_create({"amount": rest_amount}, cred_id=order_db.cred_id)
+            await models.Fiat.update_or_create({"amount": rest_amount * 100}, cred_id=order_db.cred_id)
             # отправляем продавцу чек
             if res := self.agent_client.api.upload_chat_file(upload_file=f"tmp/{dest}.png").get("result"):
                 self.agent_client.api.send_chat_message(
                     orderId=str(order_db.exid), contentType="pic", message=res["url"], msgUuid=uuid4().hex
                 )
-            logging.info(f"Order {order_db.exid} PAID at {datetime.now()}: {int_am}!")
+            logging.info(f"Order {order_db.exid} PAID at {datetime.now()}: {fmt_am}!")
 
     async def get_pma_by_cdex(self, order: OrderFull) -> tuple[PmAgentClient | None, CredEx] | None:
         cdxs = await models.CredEx.filter(
