@@ -4,7 +4,6 @@ import re
 from asyncio import sleep, gather
 from collections import defaultdict
 from datetime import datetime, timedelta, timezone
-from decimal import Decimal
 from difflib import SequenceMatcher
 from enum import IntEnum
 from http.client import HTTPException
@@ -15,9 +14,6 @@ from asyncpg import ConnectionDoesNotExistError
 from bybit_p2p import P2P
 from bybit_p2p._exceptions import FailedRequestError
 from pyro_client.client.file import FileClient
-from pyro_client.client.filler import cond_start_handler
-from pyrogram.filters import command
-from pyrogram.handlers import MessageHandler
 from tortoise import BaseDBAsyncClient
 from tortoise.exceptions import IntegrityError
 from tortoise.expressions import F, Q
@@ -428,6 +424,10 @@ class AgentClient(BaseAgentClient):  # Bybit client
     async def create_order(self, order: OrderFull) -> models.Order:
         ad = Ad(**self.api.get_ad_details(itemId=order.itemId)["result"])
         await sleep(1)
+        curex = await models.CurEx.get_or_none(ex=self.ex_client.ex, exid=order.currencyId).prefetch_related("cur")
+        cur_scale = (curex.scale if curex.scale is not None else curex.cur.scale) if curex else 2
+        coinex = await models.CoinEx.get(ex=self.ex_client.ex, exid=order.tokenId).prefetch_related("coin")
+        coin_scale = coinex.scale if coinex.scale is not None else coinex.cur.scale
         maker_name = order.buyerRealName, order.sellerRealName
         im_maker = int(order.makerUserId == order.userId)
         taker_id = (order.userId, order.targetUserId)[im_maker]
@@ -440,6 +440,8 @@ class AgentClient(BaseAgentClient):  # Bybit client
         if not ad_db:
             ...
         ecredex: CredEpyd = order.confirmedPayTerm
+        if ecredex.paymentType == 0 and im_maker and order.side:
+            ecredex = order.paymentTermList[0]
         if ecredex.paymentType:
             if not (credex := await models.CredEx.get_or_none(exid=ecredex.id, ex=self.ex_client.ex)):
                 # cur_id = await Cur.get(ticker=ad.currencyId).values_list('id', flat=True)
@@ -491,7 +493,8 @@ class AgentClient(BaseAgentClient):  # Bybit client
             logging.error(e)
         odb, _ = await models.Order.update_or_create(
             {
-                "amount": order.amount,
+                "amount": float(order.amount) * 10**cur_scale,
+                "quantity": float(order.quantity) * 10**coin_scale,
                 "status": OrderStatus[Status(order.status).name],
                 "created_at": int(order.createDate[:-3]),
                 "payed_at": order.transferDate != "0" and int(order.transferDate[:-3]) or None,
@@ -518,17 +521,17 @@ class AgentClient(BaseAgentClient):  # Bybit client
         try:
             lst = self.api.get_orders(
                 page=page,
-                size=30,
                 # status=status,  # 50 - завершено
                 # tokenId=token_id,
-                # beginTime=begin_time,
-                # endTime=end_time,
-                # side=side,  # 1 - продажа, 0 - покупка
+                beginTime=begin_time,
+                endTime=end_time,
+                # side=side, # 1 - продажа, 0 - покупка
+                size=30,
             )
         except FailedRequestError as e:
             if e.status_code == 10000:
                 await sleep(9)
-                await self.get_api_orders(page, begin_time, end_time, status, side, token_id)
+                await self.get_api_orders(page, begin_time, end_time)  # , status, side, token_id)
         ords = {int(o["id"]): OrderItem.model_validate(o) for o in lst["result"]["items"]}
         for oid, o in ords.items():
             fo = self.api.get_order_details(orderId=o.id)
@@ -537,8 +540,8 @@ class AgentClient(BaseAgentClient):  # Bybit client
             await sleep(1)
             dmsgs = self.api.get_chat_messages(orderId=oid, size=200)["result"]["result"][::-1]
             msgs = [Message.model_validate(m) for m in dmsgs if m["msgType"] in (1, 2, 7, 8)]
-            if order_db.ad.remark:
-                msgs.pop(0)
+            if order_db.ad.auto_msg:
+                msgs and msgs.pop(0)
             msgs_db = [
                 models.Msg(
                     order=order_db,
@@ -577,14 +580,14 @@ class AgentClient(BaseAgentClient):  # Bybit client
         print("-" if mad.side else "+", end=req.price, flush=True)
         await sleep(60)
 
-    def overprice_filter(self, ads: list[Ad], ceil: Decimal, k: Literal[-1, 1]):
+    def overprice_filter(self, ads: list[Ad], ceil: float, k: Literal[-1, 1]):
         # вырезаем ads с ценами выше потолка
-        if ads and (ceil - Decimal(ads[0].price)) * k > 0:
+        if ads and (ceil - float(ads[0].price)) * k > 0:
             if int(ads[0].userId) != self.actor.exid:
                 ads.pop(0)
                 self.overprice_filter(ads, ceil, k)
 
-    def get_cad(self, ads: list[Ad], ceil: Decimal, k: Literal[-1, 1], target_place: int, cur_plc: int) -> Ad:
+    def get_cad(self, ads: list[Ad], ceil: float, k: Literal[-1, 1], target_place: int, cur_plc: int) -> Ad:
         if not ads:
             return None
         # чью цену будем обгонять, предыдущей или слещующей объявы?
@@ -594,7 +597,7 @@ class AgentClient(BaseAgentClient):  # Bybit client
             logging.error(f"target place {target_place} not found in ads list {ads}")
         cad: Ad = ads[target_place]
         # а цена обгоняемой объявы не выше нашего потолка?
-        if (Decimal(cad.price) - ceil) * k <= 0:
+        if (float(cad.price) - ceil) * k <= 0:
             # тогда берем следующую
             ads.pop(target_place)
             cad = self.get_cad(ads, ceil, k, target_place, cur_plc)
@@ -626,7 +629,7 @@ class AgentClient(BaseAgentClient):  # Bybit client
         k = (-1) ** int(taker_side)  # on_buy=1, on_sell=-1
         sleep_sec = 3  # 1 if set(pms) & {"volet"} and coinex.coin_id == 1 else 5
         creds: list[models.CredEx] = await self.get_credexs_by_pms(race.road.ad.pms, curex.cur_id)
-        lstat, volume = None, 0
+        _lstat, volume = None, 0
 
         while self.actor.person.user.status > 0:
             # обновляем все обновления по текущей гонке из бд
@@ -638,7 +641,7 @@ class AgentClient(BaseAgentClient):  # Bybit client
             expiration = datetime.now(timezone.utc) - timedelta(minutes=15)
             if race.updated_at < expiration:
                 ceils, hp, vmf, zplace = await self.get_ceils(coinex, curex, pmexs, 0.001, True)
-                race.ceil = ceils[int(taker_side)]
+                race.ceil = int(ceils[int(taker_side)] * 10**curex.scale)
                 await race.save()
                 # upd pair race
                 if prace := await models.Race.annotate(pms_count=Count("road__ad__pms")).get_or_none(
@@ -649,7 +652,7 @@ class AgentClient(BaseAgentClient):  # Bybit client
                     road__ad__pms__id__in=pm_ids,
                     pms_count=len(pm_ids),
                 ):
-                    prace.ceil = ceils[int(not taker_side)]
+                    prace.ceil = int(ceils[int(not taker_side)] * 10**curex.scale)
                     await prace.save()
 
             last_vol = volume
@@ -658,13 +661,13 @@ class AgentClient(BaseAgentClient):  # Bybit client
                 volume = fiat.amount / race.road.ad.price
             else:  # гонка в стакане покупки - мы продаем МОНЕТУ за фиат
                 asset = await models.Asset.get(addr__actor=self.actor, addr__coin_id=coinex.coin_id)
-                volume = asset.free - (asset.freeze or 0) - (asset.lock or 0)
+                volume = (asset.free - (asset.freeze or 0) - (asset.lock or 0)) * 10**-coinex.scale
             volume = str(round(volume, coinex.scale))
 
             ads: list[Ad] = await self.ads(coinex, curex, taker_side, pmexs)
             if race.vm_filter:
                 ads = [ad for ad in ads if "VA" in ad.authTag]
-            self.overprice_filter(ads, race.ceil, k)  # обрезаем сверху все ads дороже нашего потолка
+            self.overprice_filter(ads, race.ceil * 10**-curex.scale, k)  # обрезаем сверху все ads дороже нашего потолка
 
             if 571 in pm_ids and coinex.coin.ticker == "USDT" and not taker_side:
                 ...
@@ -679,53 +682,61 @@ class AgentClient(BaseAgentClient):  # Bybit client
                 await sleep(15)
                 continue
             (cur_plc,) = cur_plc  # может упасть если в списке > 1 наш ad
-            rads = [(await self.cond_upsert(ad, ps=race.road.ad.pair_side, force=True))[0] for ad in ads[:cur_plc]]
-            rivals = [
-                (await models.Rival.update_or_create({"place": plc, "price": ad.price, "premium": ad.premium}, ad=ad))[
-                    0
-                ]
-                for plc, ad in enumerate(rads)
-            ]
+            [(await self.cond_upsert(ad, ps=race.road.ad.pair_side, force=True))[0] for ad in ads[:cur_plc]]
+            # rivals = [
+            #     (await models.RaceStat.update_or_create({"place": plc, "price": ad.price, "premium": ad.premium}, ad=ad))[
+            #         0
+            #     ]
+            #     for plc, ad in enumerate(rads)
+            # ]
             mad: Ad = ads.pop(cur_plc)
-            if (
-                not (lstat := lstat or await race.stats.order_by("-created_at").first())
-                or lstat.place != cur_plc
-                or lstat.price != Decimal(mad.price)
-                or set(rivals) != set(await lstat.rivals)
-            ):
-                lstat = await models.RaceStat.create(race=race, place=cur_plc, price=mad.price, premium=mad.premium)
-                await lstat.rivals.add(*rivals)
+            # if (
+            #     not (lstat := lstat or await race.stats.order_by("-created_at").first())
+            #     or lstat.place != cur_plc
+            #     or lstat.price != float(mad.price)
+            #     or set(rivals) != set(await lstat.rivals)
+            # ):
+            #     lstat = await models.RaceStat.create(race=race, place=cur_plc, price=mad.price, premium=mad.premium)
+            #     await lstat.rivals.add(*rivals)
             if not ads:
                 await sleep(60)
                 continue
-            if not (cad := self.get_cad(ads, race.ceil, k, race.target_place, cur_plc)):
+            if not (cad := self.get_cad(ads, race.ceil * 10**-curex.scale, k, race.target_place, cur_plc)):
                 continue
-            new_price = Decimal(cad.price) - k * step(mad, cad, curex.cur.scale)
+            new_price = round(float(cad.price) - k * step(mad, cad, curex.cur.scale), curex.cur.scale)
             if (
-                Decimal(mad.price) == new_price and volume == last_vol
+                float(mad.price) == new_price and volume == last_vol
             ):  # Если место уже нужное или нужная цена и так уже стоит
-                print(f"{'v' if taker_side else '^'}{mad.price}", end=f"[{race.ceil}+{cur_plc}] ", flush=True)
+                print(
+                    f"{'v' if taker_side else '^'}{mad.price}",
+                    end=f"[{race.ceil * 10**-curex.scale}+{cur_plc}] ",
+                    flush=True,
+                )
                 await sleep(sleep_sec)
                 continue
             if cad.priceType:  # Если цена конкурента плавающая, то повышаем себе не цену, а %
-                new_premium = Decimal(cad.premium) - k * step(mad, cad, 2)
-                if Decimal(mad.premium) == new_premium:  # Если нужный % и так уже стоит
+                new_premium = float(cad.premium) - k * step(mad, cad, 2)
+                if float(mad.premium) == new_premium:  # Если нужный % и так уже стоит
                     if mad.priceType and cur_plc != race.target_place:
                         new_premium -= k * step(mad, cad, 2)
                     elif volume == last_vol:
                         print(end="v" if taker_side else "^", flush=True)
                         await sleep(sleep_sec)
                         continue
-                mad.premium = new_premium.to_eng_string()
+                mad.premium = str(round(new_premium, 2))
             mad.priceType = cad.priceType
             mad.quantity = volume
             mad.maxAmount = str(2_000_000)
             req = AdUpdateRequest.model_validate(
-                {**mad.model_dump(), "price": new_price.to_eng_string(), "paymentIds": [str(p.exid) for p in creds]}
+                {
+                    **mad.model_dump(),
+                    "price": str(round(new_price, curex.scale)),
+                    "paymentIds": [str(p.exid) for p in creds],
+                }
             )
             try:
                 print(
-                    f"c{race.ceil}+{cur_plc} {coinex.coin.ticker}{'-' if taker_side else '+'}{req.price}{curex.cur.ticker}"
+                    f"c{race.ceil * 10**-curex.scale}+{cur_plc} {coinex.coin.ticker}{'-' if taker_side else '+'}{req.price}{curex.cur.ticker}"
                     f"{[pm.norm for pm in race.road.ad.pms]}{f'({req.premium}%)' if req.premium != '0' else ''} "
                     f"t{race.target_place} ;",
                     flush=True,
@@ -745,7 +756,10 @@ class AgentClient(BaseAgentClient):  # Bybit client
                 elif ExcCode(e.status_code) == ExcCode.InsufficientBalance:
                     asset = await models.Asset.get(addr__actor=self.actor, addr__coin_id=coinex.coin_id)
                     req.quantity = str(
-                        round(asset.free - (asset.freeze or 0) - (asset.lock or 0), coinex.coin.scale or coinex.scale)
+                        round(
+                            (asset.free - (asset.freeze or 0) - (asset.lock or 0)) * 10**-coinex.scale,
+                            coinex.coin.scale or coinex.scale,
+                        )
                     )
                     _res = self.ad_upd(req)
                 elif ExcCode(e.status_code) == ExcCode.RareLimit:
@@ -780,9 +794,11 @@ class AgentClient(BaseAgentClient):  # Bybit client
         return buy, sell
 
     async def get_spread(
-        self, bb: list[Ad], sb: list[Ad], perc: Decimal, vmf: bool = None, place: int = 0, exact: bool = False
-    ) -> tuple[tuple[Decimal, Decimal], Decimal, bool, int]:
-        buy_price, sell_price = Decimal(bb[place].price), Decimal(sb[place].price)
+        self, bb: list[Ad], sb: list[Ad], perc: float, vmf: bool = None, place: int = 0, exact: bool = False
+    ) -> tuple[tuple[float, float], float, bool, int]:
+        if len(bb) <= place or len(sb) <= place:
+            ...
+        buy_price, sell_price = float(bb[place].price), float(sb[place].price)
         half_spread = (buy_price - sell_price) / (buy_price + sell_price)
         if half_spread * 2 < perc:
             if not exact:
@@ -798,16 +814,15 @@ class AgentClient(BaseAgentClient):  # Bybit client
         coinex: models.CoinEx,
         curex: models.CurEx,
         pmexs: list[models.PmEx],
-        min_prof=0.01,
+        min_prof=0.02,
         vmf: bool = False,
         place: int = 0,
-    ) -> tuple[tuple[Decimal, Decimal], Decimal, bool, int]:  # todo: refact to Pairex
-        min_prof = Decimal(min_prof)
+    ) -> tuple[tuple[float, float], float, bool, int]:  # todo: refact to Pairex
         bb, sb = await self.get_books(coinex, curex, pmexs)
         if vmf:
             bb = [b for b in bb if "VA" in b.authTag]
             sb = [s for s in sb if "VA" in s.authTag]
-        perc = pmexs[0].pm.fee + min_prof
+        perc = pmexs[0].pm.fee * 0.0001 + min_prof
         (bf, sf), hp, vmf, zplace = await self.get_spread(bb, sb, perc, vmf, place)
         mdl = (bf + sf) / 2
         bc, sc = mdl + mdl * (perc / 2), mdl - mdl * (perc / 2)
@@ -921,7 +936,7 @@ class AgentClient(BaseAgentClient):  # Bybit client
             return ad_db, False
 
         cid = await self.cond_new(cleaned, {int(ad.userId)})
-        return await self.ad_create(ad, cid), True
+        return await self.ad_create(ad, cid, rname, ps), True
 
     async def cond_new(self, txt: str, uids: set[int]) -> int:
         new_cond, _ = await Cond.update_or_create(raw_txt=txt)
@@ -993,6 +1008,7 @@ class AgentClient(BaseAgentClient):  # Bybit client
         if rname:
             act_df |= {"person": await self.person_upsert(rname, int(ad.userId))}
         if not actor:
+            act_df |= {"person": await self.person_upsert(ad.nickName, int(ad.userId))}
             actor, _ = await Actor.update_or_create(act_df, exid=ad.userId, ex=self.ex_client.ex)
         ps = ps or await PairSide.get_or_none(
             is_sell=ad.side,
@@ -1002,16 +1018,17 @@ class AgentClient(BaseAgentClient):  # Bybit client
         if not ps or not ps.pair:
             ...  # THB/USDC
         ad_upd = models.Ad.validate(ad.model_dump(by_alias=True))
-        amount = int(float(ad.quantity) * float(ad.price) * 10**ps.pair.cur.scale)
+        cur_scale = 10**ps.pair.cur.scale
+        coinex = await models.CoinEx.get(coin_id=ps.pair.coin_id, ex=self.ex_client.ex)
         df_unq = ad_upd.df_unq(
             maker_id=actor.id,
             pair_side_id=ps.id,
-            amount=amount,
-            quantity=int(float(ad.quantity) * 10**ps.pair.coin.scale),
-            min_fiat=int(float(ad.minAmount) * 10**ps.pair.cur.scale),
-            max_fiat=ad.maxAmount and int(float(ad.maxAmount) * 10**ps.pair.cur.scale),
-            price=int(float(ad.price) * 10**ps.pair.cur.scale),
-            premium=int(float(ad.premium) * 10**ps.pair.cur.scale),
+            amount=int(float(ad.quantity) * float(ad.price) * cur_scale),
+            quantity=int(float(ad.quantity) * 10**coinex.scale),
+            min_fiat=int(float(ad.minAmount) * cur_scale),
+            max_fiat=ad.maxAmount and int(float(ad.maxAmount) * cur_scale),
+            price=int(float(ad.price) * cur_scale),
+            premium=int(float(ad.premium) * cur_scale),
         )
         ad_db, _ = await models.Ad.update_or_create(**df_unq)
         await ad_db.pms.add(*(await models.Pm.filter(pmexs__ex=self.ex_client.ex, pmexs__exid__in=ad.payments)))
@@ -1135,8 +1152,8 @@ def step_is_need(mad, cad) -> bool:
     )
 
 
-def step(mad, cad, scale: int = 2) -> Decimal:
-    return Decimal(int(step_is_need(mad, cad)) * 10**-scale).__round__(scale)
+def step(mad, cad, scale: int = 2) -> float:
+    return float(int(step_is_need(mad, cad)) * 10**-scale).__round__(scale)
 
 
 class ExcCode(IntEnum):
@@ -1146,6 +1163,7 @@ class ExcCode(IntEnum):
     Timestamp = 10002
     IP = 10010
     Quantity = 912300019
+    Unknown = 912300014
 
 
 async def main():
@@ -1168,21 +1186,28 @@ async def main():
     )
     async with FileClient(TOKEN) as b:
         b: FileClient
-        b.add_handler(MessageHandler(cond_start_handler, command("cond")))
+        # b.add_handler(MessageHandler(cond_start_handler, command("cond")))
         cl: AgentClient = actor.client(b)
 
-        await cl.ex_client.set_pairs()
-        await cl.export_my_ads()
+        # await cl.ex_client.set_pairs()
+        # await cl.ex_client.set_pms()
+        # await cl.set_creds()
+        # await cl.export_my_ads()
 
-        await cl.ex_client.set_pms(cookies=actor.agent.auth["cookies"])  # 617 -> 639
-        await cl.set_creds()
+        # создание гонок по мои активным объявам:
+        # for ma in cl.my_ads():
+        #     my_ad = await models.MyAd.get(ad__exid=ma.id).prefetch_related('ad__pms', 'ad__pair_side__pair')
+        #     race, _ = await models.Race.update_or_create(
+        #         {"started": True, "vm_filter": True, "target_place": 5},
+        #         road=my_ad
+        #     )
 
         # for name in names:
         #     s, _ = await models.Synonym.update_or_create(typ=SynonymType.name, txt=name)
         #     await s.curs.add(rub.cur)
 
         # пока порешали рейс-кондишн, очищаем сиротские условия при каждом запуске
-        [await c.delete() for c in await Cond.filter(ads__isnull=True)]
+        # [await c.delete() for c in await Cond.filter(ads__isnull=True)]
         cl.all_conds = {
             c.id: (c.raw_txt, {a.maker.exid for a in c.ads}) for c in await Cond.all().prefetch_related("ads__maker")
         }
@@ -1203,9 +1228,12 @@ async def main():
         if ct := set(cl.tree.keys()) & a:
             logging.exception(f"cycle cids: {ct}")
 
-        await cl.get_api_orders()
+        # await cl.get_api_orders(43, 1741294800000, 1749157199999)
 
-        races = await models.Race.filter(started=True).prefetch_related("road__ad__pair_side__pair", "road__ad__pms")
+        races = await models.Race.filter(started=True).prefetch_related(
+            "road__ad__pair_side__pair__cur",
+            "road__ad__pms",
+        )
         tasks = [asyncio.create_task(cl.racing(race), name=f"Rc{race.id}") for race in races]
         # await cl.actual_cond()
         await gather(
