@@ -1,4 +1,3 @@
-import logging
 from asyncio import run
 from datetime import datetime
 from decimal import Decimal
@@ -8,7 +7,7 @@ from time import sleep
 
 from asyncpg.pgproto.pgproto import timedelta
 from payeer_api import PayeerAPI
-from playwright.async_api import async_playwright, Playwright, Locator
+from playwright.async_api import async_playwright, Playwright
 from playwright._impl._errors import TimeoutError
 
 from xync_client.loader import TORM
@@ -39,16 +38,16 @@ class Client(PmAgentClient):
             await self.page.context.add_cookies([cookie])
         await self.page.goto(self.pages.SEND)
 
-    async def send(self, dest: str, amount: int, cur: str) -> tuple[int, bytes, float]:
-        self.last_active = datetime.now()
+    async def send(self, dest: str, amount: int, cur: str) -> tuple[int, bytes, int] | int:
         page = self.page
-        try:
-            await page.goto(self.pages.SEND)
-        except TimeoutError:
-            await login(self.agent)
-            for cookie in self.agent.state["cookies"]:
-                await page.context.add_cookies([cookie])
-            await page.goto("https://payeer.com/en/account/send/")
+        if not page.url.startswith(self.pages.SEND):
+            try:
+                await page.goto(self.pages.SEND)
+            except TimeoutError:
+                await login(self.agent)
+                for cookie in self.agent.state["cookies"]:
+                    await page.context.add_cookies([cookie])
+                await page.goto("https://payeer.com/en/account/send/")
         fiat_accounts = await page.locator(
             f".balance-item.balance-item--green.balance-item--{cur.lower()}"
         ).all_text_contents()
@@ -56,32 +55,41 @@ class Client(PmAgentClient):
         if float(amount) <= has_amount:
             await page.locator('input[name="param_ACCOUNT_NUMBER"]').fill(dest)
             await page.locator("select[name=curr_receive]").select_option(value=cur)
-            await fill_amount(page.locator('input[name="sum_receive"]'), str(amount))
-            await page.wait_for_selector(".btn.n-form--btn.n-form--btn-mod", state="visible")
-            await click_send(page.locator(".btn.n-form--btn.n-form--btn-mod:not(.repeat)"))
+            sleep(0.9)
+            await page.locator('input[name="sum_receive"]').fill(str(amount))
+            sleep(0.1)
+            # await page.locator("div.n-form--title").first.click()
+            # sleep(0.1)
+            await page.click(".btn.n-form--btn.n-form--btn-mod")
+            sleep(0.5)
+            await page.click(".btn.n-form--btn.n-form--btn-mod")
+            sleep(1.2)
+            if await page.locator(".input4").count():
+                await page.locator(".input4").fill(self.agent.auth.get("master_key"))
+                await page.click(".ok.button_green2")
+            sleep(0.4)
             await page.locator(".note_txt").wait_for(state="visible")
             if await page.locator('.note_txt:has-text("successfully completed")').count():
                 transaction = await page.locator(".note_txt").all_text_contents()
                 trans_num = int(transaction[0].replace("Transaction #", "").split()[0])
                 await page.goto("https://payeer.com/ru/account/history/")
                 await page.click(f".history-id-{trans_num} a.link")
-                await page.wait_for_selector(".ui-dialog.ui-corner-all", state="visible")
+                sleep(1)
                 receipt = await page.query_selector(".ui-dialog.ui-corner-all")
-                return trans_num, await receipt.screenshot(path=f"tmp/{dest}.png"), has_amount
+                return trans_num, await receipt.screenshot(path=f"tmp/{dest}.png"), has_amount - amount
             else:
                 await self.bot.send("Payeer хз", self.uid, photo=await self.page.screenshot())
-                raise Exception(dest, amount, cur)
+                return -1
         else:
-            have_amount = int(fiat_accounts[0].strip())
             await self.bot.send(
-                f"Payeer no have {amount}, only {have_amount}{cur} to {dest}",
+                f"Payeer no have {amount}, only {has_amount}{cur} to {dest}",
                 self.uid,
                 photo=await self.page.screenshot(),
             )
-            raise Exception(f"Need {amount}{cur}, but has only {has_amount}")
+            return has_amount
 
     def check_in(
-        self, amount: Decimal | int | float, cur: str, time: datetime = None, tid: str | int = None
+        self, amount: Decimal | int | float, cur: str, tme: datetime = None, tid: str | int = None
     ) -> tuple[Decimal | None, int | None]:
         history = self.api.history(type="incoming", append=tid, count=3)
         if tid:
@@ -92,7 +100,7 @@ class Client(PmAgentClient):
             if (
                 amount <= Decimal(h["creditedAmount"]) <= ceil(amount)
                 and h["creditedCurrency"] == cur
-                and datetime.fromisoformat(h["date"]) > time - timedelta(minutes=1)
+                and datetime.fromisoformat(h["date"]) > tme - timedelta(minutes=1)
             )
         ]
         if not (t := t and t[0]):
@@ -104,30 +112,6 @@ class Client(PmAgentClient):
     async def proof(self) -> bytes: ...
 
 
-async def fill_amount(loc: Locator, amount: str):
-    await loc.wait_for(state="visible")
-    await loc.fill(amount)
-    sleep(0.4)
-    if await loc.input_value() != amount or await loc.input_value() != amount:
-        logging.warning("Fill amount repeated!")
-        await fill_amount(loc, amount)
-    else:
-        ...
-
-
-async def click_send(loc: Locator, count: int = 1):
-    if await loc.is_visible():
-        await loc.click(delay=70)
-        try:
-            await loc.wait_for(state="hidden", timeout=900 * count)
-        except TimeoutError:
-            if count < 4:
-                logging.warning("Click repeated!")
-                await click_send(loc, count + 1)
-            else:
-                raise TimeoutError
-
-
 async def main(uid: int):
     from x_model import init_db
 
@@ -139,8 +123,13 @@ async def main(uid: int):
 
         dest, amount, cur = "P79619335", 2, "RUB"
 
-        res = pyr.check_in(3731, cur, datetime.now())
         res = await pyr.send(dest, amount, cur)
+        res = await pyr.send(dest, 3, cur)
+        res = await pyr.send(dest, amount, cur)
+        res = await pyr.send(dest, 3, cur)
+        res = await pyr.send(dest, amount, cur)
+
+        res = pyr.check_in(2, cur, datetime.now())
 
         if len(res) > 1 and isinstance(res[1], bytes):
             await pyr.bot.send(f"Transaction #{res[0]}", uid, photo=res[1])
