@@ -1,51 +1,36 @@
-from time import sleep
-
 from selenium.webdriver.common.by import By
 import undetected_chromedriver as uc
 from xync_schema.models import PmAgent
 from selenium.webdriver.support.ui import WebDriverWait
 from selenium.webdriver.support import expected_conditions as EC
 from selenium.webdriver.common.action_chains import ActionChains
+import time
 
 
 async def login(agent: PmAgent):
-    driver = uc.Chrome(no_sandbox=True)
+    driver = uc.Chrome(no_sandbox=True, headless=False)
     wait = WebDriverWait(driver, timeout=10)
     try:
-        driver.get("https://payeer.com/ru/auth")
+        driver.get("https://payeer.com/en/auth")
         wait.until(EC.invisibility_of_element_located((By.TAG_NAME, "lottie-player")))
         login_link = wait.until(EC.element_to_be_clickable((By.CLASS_NAME, "button.button_empty")))
-        try:
-            login_link.click()
-        except Exception:
-            driver.execute_script("arguments[0].click();", login_link)
+        login_link.click()
         email_field = wait.until(EC.presence_of_element_located((By.NAME, "email")))
         email_field.send_keys(agent.auth.get("email"))
         password_field = wait.until(EC.presence_of_element_located((By.NAME, "password")))
         password_field.send_keys(agent.auth.get("password"))
         login_button = wait.until(EC.element_to_be_clickable((By.CLASS_NAME, "login-form__login-btn.step1")))
-        sleep(1)
-        try:
-            login_button.click()
-        except Exception:
-            driver.execute_script("arguments[0].click();", login_button)
-        try:
-            login_button = wait.until(EC.element_to_be_clickable((By.CLASS_NAME, "login-form__login-btn.step1")))
-            driver.execute_script("arguments[0].click();", login_button)
-        except Exception:
-            pass
+        login_button.click()
+        time.sleep(5)
+        login_button.click()
+        time.sleep(1)
         if (v := driver.find_elements(By.CLASS_NAME, "form-input-top")) and v[0].text == "Введите проверочный код":
             code = input("Email code: ")
             actions = ActionChains(driver)
             for char in code:
                 actions.send_keys(char).perform()
             step2_button = wait.until(EC.element_to_be_clickable((By.CLASS_NAME, "login-form__login-btn.step2")))
-            try:
-                step2_button.click()
-            except Exception:
-                driver.execute_script("arguments[0].click();", step2_button)
-
-        sleep(1)
+            step2_button.click()
         agent.state = {"cookies": driver.get_cookies()}
         await agent.save()
     finally:
